/* -*- mode: c ; c-file-style: "canonware-c-style" -*-
 ******************************************************************************
 *
 * Copyright (C) 1996-2004 Jason Evans <jasone@canonware.com>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice(s), this list of conditions and the following disclaimer
 *    unmodified other than the allowable addition of one or more
 *    copyright notices.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice(s), this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 *
 * Version: Onyx 5.1.1
 *
 * Test the mq (message queue) class.
 *
 ******************************************************************************/

#include "../include/libonyx/libonyx.h"

int
main()
{
    libonyx_init(0, NULL, NULL);
    fprintf(stderr, "Test begin\n");

    /* mq_new(), mq_delete(). */
    {
	cw_mq_t mq;

	mq_new(&mq, cw_g_mema, sizeof(uint32_t));
	mq_delete(&mq);

	mq_new(&mq, cw_g_mema, sizeof(uint64_t));
	mq_delete(&mq);
    }

    /* mq_tryget(), mq_put(). */
    {
	cw_mq_t mq;
	uint8_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint8_t));

	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == k);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	mq_delete(&mq);
    }
    {
	cw_mq_t mq;
	uint16_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint16_t));

	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == k);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	mq_delete(&mq);
    }
    {
	cw_mq_t mq;
	uint32_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint32_t));

	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == k);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	mq_delete(&mq);
    }
    {
	cw_mq_t mq;
	uint64_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint64_t));

	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == k);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	mq_delete(&mq);
    }

    /* mq_timedget(), mq_put(). */
    {
	cw_mq_t mq;
	uint8_t i = 1, j = 2, k = 3, t;
	struct timespec timeout = {0, 10000};

	mq_new(&mq, cw_g_mema, sizeof(uint8_t));

	cw_assert(mq_timedget(&mq, &timeout));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_timedget(&mq, &timeout));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == k);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_timedget(&mq, &timeout, &t));

	mq_delete(&mq);
    }
    {
	cw_mq_t mq;
	uint16_t i = 1, j = 2, k = 3, t;
	struct timespec timeout = {0, 10000};

	mq_new(&mq, cw_g_mema, sizeof(uint16_t));

	cw_assert(mq_timedget(&mq, &timeout));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_timedget(&mq, &timeout));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == k);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_timedget(&mq, &timeout, &t));

	mq_delete(&mq);
    }
    {
	cw_mq_t mq;
	uint32_t i = 1, j = 2, k = 3, t;
	struct timespec timeout = {0, 10000};

	mq_new(&mq, cw_g_mema, sizeof(uint32_t));

	cw_assert(mq_timedget(&mq, &timeout));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_timedget(&mq, &timeout));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == k);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_timedget(&mq, &timeout, &t));

	mq_delete(&mq);
    }
    {
	cw_mq_t mq;
	uint64_t i = 1, j = 2, k = 3, t;
	struct timespec timeout = {0, 10000};

	mq_new(&mq, cw_g_mema, sizeof(uint64_t));

	cw_assert(mq_timedget(&mq, &timeout));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_timedget(&mq, &timeout));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == k);
	cw_assert(mq_timedget(&mq, &timeout, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_timedget(&mq, &timeout, &t));

	mq_delete(&mq);
    }

    /* mq_get(), mq_put(), mq_tryget(). */
    {
	cw_mq_t mq;
	uint8_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint8_t));

	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == k);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	mq_delete(&mq);
    }
    {
	cw_mq_t mq;
	uint16_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint16_t));

	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == k);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	mq_delete(&mq);
    }
    {
	cw_mq_t mq;
	uint32_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint32_t));

	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == k);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	mq_delete(&mq);
    }
    {
	cw_mq_t mq;
	uint64_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint64_t));

	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == k);
	cw_assert(mq_get(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	mq_delete(&mq);
    }

    /* mq_get_start(), mq_get_stop(), mq_tryget(), mq_put(). */
    {
	cw_mq_t mq;
	uint8_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint8_t));

	cw_assert(mq_get_stop(&mq) == false);
	cw_assert(mq_get_stop(&mq));
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_get_start(&mq) == false);
	cw_assert(mq_get_start(&mq));
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);

	cw_assert(mq_get_stop(&mq) == false);
	cw_assert(mq_get_stop(&mq));
	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_get(&mq, &t));

	cw_assert(mq_get_start(&mq) == false);
	cw_assert(mq_get_start(&mq));
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);

	mq_delete(&mq);
    }
    {
	cw_mq_t mq;
	uint16_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint16_t));

	cw_assert(mq_get_stop(&mq) == false);
	cw_assert(mq_get_stop(&mq));
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_get_start(&mq) == false);
	cw_assert(mq_get_start(&mq));
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);

	cw_assert(mq_get_stop(&mq) == false);
	cw_assert(mq_get_stop(&mq));
	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_get(&mq, &t));

	cw_assert(mq_get_start(&mq) == false);
	cw_assert(mq_get_start(&mq));
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);

	mq_delete(&mq);
    }
    {
	cw_mq_t mq;
	uint32_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint32_t));

	cw_assert(mq_get_stop(&mq) == false);
	cw_assert(mq_get_stop(&mq));
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_get_start(&mq) == false);
	cw_assert(mq_get_start(&mq));
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);

	cw_assert(mq_get_stop(&mq) == false);
	cw_assert(mq_get_stop(&mq));
	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_get(&mq, &t));

	cw_assert(mq_get_start(&mq) == false);
	cw_assert(mq_get_start(&mq));
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);

	mq_delete(&mq);
    }
    {
	cw_mq_t mq;
	uint64_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint64_t));

	cw_assert(mq_get_stop(&mq) == false);
	cw_assert(mq_get_stop(&mq));
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_get_start(&mq) == false);
	cw_assert(mq_get_start(&mq));
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);

	cw_assert(mq_get_stop(&mq) == false);
	cw_assert(mq_get_stop(&mq));
	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_get(&mq, &t));

	cw_assert(mq_get_start(&mq) == false);
	cw_assert(mq_get_start(&mq));
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);
	cw_assert(mq_put(&mq, k) == 0);

	mq_delete(&mq);
    }

    /* mq_put_start(), mq_put_stop(), mq_tryget(), mq_put(). */
    {
	cw_mq_t mq;
	uint8_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint8_t));

	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_put_stop(&mq) == false);
	cw_assert(mq_put_stop(&mq));
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(1 == mq_put(&mq, i));

	cw_assert(mq_put_start(&mq) == false);
	cw_assert(mq_put_start(&mq));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);

	cw_assert(mq_put_stop(&mq) == false);
	cw_assert(mq_put_stop(&mq));
	cw_assert(mq_put(&mq, k) == 1);

	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_tryget(&mq, &t));

	mq_delete(&mq);
    }
    {
	cw_mq_t mq;
	uint16_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint16_t));

	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_put_stop(&mq) == false);
	cw_assert(mq_put_stop(&mq));
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(1 == mq_put(&mq, i));

	cw_assert(mq_put_start(&mq) == false);
	cw_assert(mq_put_start(&mq));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);

	cw_assert(mq_put_stop(&mq) == false);
	cw_assert(mq_put_stop(&mq));
	cw_assert(mq_put(&mq, k) == 1);

	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_tryget(&mq, &t));

	mq_delete(&mq);
    }
    {
	cw_mq_t mq;
	uint32_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint32_t));

	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_put_stop(&mq) == false);
	cw_assert(mq_put_stop(&mq));
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(1 == mq_put(&mq, i));

	cw_assert(mq_put_start(&mq) == false);
	cw_assert(mq_put_start(&mq));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);

	cw_assert(mq_put_stop(&mq) == false);
	cw_assert(mq_put_stop(&mq));
	cw_assert(mq_put(&mq, k) == 1);

	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_tryget(&mq, &t));

	mq_delete(&mq);
    }
    {
	cw_mq_t mq;
	uint64_t i = 1, j = 2, k = 3, t;

	mq_new(&mq, cw_g_mema, sizeof(uint64_t));

	cw_assert(mq_tryget(&mq, &t));
	cw_assert(mq_put_stop(&mq) == false);
	cw_assert(mq_put_stop(&mq));
	cw_assert(mq_tryget(&mq, &t));

	cw_assert(1 == mq_put(&mq, i));

	cw_assert(mq_put_start(&mq) == false);
	cw_assert(mq_put_start(&mq));
	cw_assert(mq_put(&mq, i) == 0);
	cw_assert(mq_put(&mq, j) == 0);

	cw_assert(mq_put_stop(&mq) == false);
	cw_assert(mq_put_stop(&mq));
	cw_assert(mq_put(&mq, k) == 1);

	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == i);
	cw_assert(mq_tryget(&mq, &t) == false);
	cw_assert(t == j);
	cw_assert(mq_tryget(&mq, &t));

	mq_delete(&mq);
    }

    fprintf(stderr, "Test end\n");
    libonyx_shutdown();
    return 0;
}
